<?php

/**
 * This is a postit.lt post code API integration module for Prestashop. 
 *
 * @author    UAB Euro Info
 * @copyright 2019
 * @license   MIT License
 */
if (!defined('_PS_VERSION_')) {
	exit;
}

class Postit extends Module
{
	
	const MODULE_NAME = 'POSTIT';
	const URL_V1 = 'https://api.postit.lt/';
	const URL_CHECK = 'https://api.postit.lt/check/{KEY}';

	protected $trial_text = '<a href="https://postit.lt/" target="_blank" title="Pašto kodų paieška">Pašto kodų paieška</a>';
	protected $default_options = [
		'api_type' => 'v1_1',
		'prefix' => '',
		'label_suffix' => ' »',
		'input_country' => 'select[name="id_country"]',
		'input_country_lt_value' => '131',
		'input_address' => 'input[name="address1"]',
		//'input_municipality' => 'input[name="address2"]',
		'input_city' => 'input[name="city"]',
		'input_postcode' => 'input[name="postcode"]',
		'limit' => 10,
		'autocomplete_version' => 0,	//0 - auto; 1 - old; 2 - new
	];

	public function __construct()
	{
		$this->name = 'postit';
		$this->tab = 'administration';
		$this->version = '1.0.5b';
		$this->author = 'UAB Euro Info';
		$this->need_instance = 0;
		$this->ps_versions_compliancy = [
			'min' => '1.6',
			'max' => '1.7.999',
		];
		$this->bootstrap = true;

		parent::__construct();

		$this->displayName = $this->l('postit.lt integracija');
		$this->description = $this->l('Lietuvos adresų ir pašto kodų automatinė paieška');
		
		if(version_compare(_PS_VERSION_, '1.7.0') < 0){
			$this->default_options['input_country'] = '#id_country';
			$this->default_options['input_address'] = '#address1';
			$this->default_options['input_city'] = '#city';
			$this->default_options['input_postcode'] = '#postcode';
		}
	}

	/**
	 * Installs module and it's resources
	 *
	 * @return bool
	 */
	public function install()
	{
		$isError = false;
		$errorMessage = '';

		if (!parent::install()) {
			$isError = true;
			$errorMessage = 'Could not install new module.';
		}
		elseif (!$this->installDefaultValues()) {
			$isError = true;
			$errorMessage = 'Failed to install default values.';
		}
		elseif (!$this->registerHook('header')) {
			$isError = true;
			$errorMessage = 'Failed to register header hook.';
		}

		if ($isError) {
			$this->_errors[] = sprintf($this->l('An error occurred during installation process: %s'), (__CLASS__ . ': ' . $this->l($errorMessage)));
			$this->uninstall();
			return false;
		}

		return true;
	}

	/**
	 * Uninstalls module and it's resources
	 *
	 * @return bool
	 */
	public function uninstall()
	{
		$isError = false;
		$errorMessage = '';

		if (!parent::uninstall()) {
			$isError = true;
			$errorMessage = 'Could not uninstall module.';
		}

		if ($isError) {
			$this->_errors[] = sprintf($this->l('An error occurred during uninstallation process: %s'), (__CLASS__ . ': ' . $this->l($errorMessage)));
			return false;
		}

		return true;
	}

	/*
	 * Install default values to avoid notifications
	 * 
	 * @return bool
	 */
	protected function installDefaultValues()
	{
		Configuration::updateValue(self::MODULE_NAME . '_API_KEY', '');
		Configuration::updateValue(self::MODULE_NAME . '_OPTIONS', json_encode($this->default_options));

		return true;
	}

	/*
	 * Get Main module content
	 */
	public function getContent()
	{
		$output = '';
		$isSubmit = (bool)Tools::isSubmit('submit' . $this->name . 'Module');
		if ($isSubmit === true) {
			$output .= $this->postProcess();
		}

		$this->context->smarty->assign([
			'module_dir' => $this->_path,
		]);

		$output .= $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');
		$output .= $this->renderForm();

		return $output;
	}

	/*
	 * Render configuration form
	 */
	protected function renderForm()
	{
		$helper = new HelperForm();

		$helper->show_toolbar = false;
		$helper->table = $this->table;
		$helper->module = $this;
		$helper->default_form_language = $this->context->language->id;
		$helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

		$helper->identifier = $this->identifier;
		$helper->submit_action = 'submit' . $this->name . 'Module';
		$helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');

		$helper->tpl_vars = [
			'fields_value' => $this->getConfigFormValues(),
			'languages' => $this->context->controller->getLanguages(),
			'id_language' => $this->context->language->id,
		];

		return $helper->generateForm([$this->getConfigForm()]);
	}
	
	protected function getConfigForm()
	{
		$inputs = [
			[
				'type' => 'text',
				'col' => 4,
				'required' => false,
				'label' => $this->l('API key'),
				'name' => self::MODULE_NAME . '_API_KEY',
			],
			[
				'type' => 'text',
				'col' => 4,
				'required' => false,
				'label' => $this->l('Post code prefix'),
				'name' => self::MODULE_NAME . '_OPTIONS[prefix]',
				'suffix' => $this->inputSuffix(isset($this->default_options['prefix']) ? $this->default_options['prefix'] : null),
			],
			[
				'type' => 'text',
				'col' => 4,
				'required' => false,
				'label' => $this->l('Address suggestion list label suffix'),
				'name' => self::MODULE_NAME . '_OPTIONS[label_suffix]',
				'suffix' => $this->inputSuffix(isset($this->default_options['label_suffix']) ? $this->default_options['label_suffix'] : null),
			],
			[
				'type' => 'text',
				'col' => 4,
				'required' => false,
				'label' => $this->l('Suggestion list limit'),
				'name' => self::MODULE_NAME . '_OPTIONS[limit]',
				'suffix' => $this->inputSuffix(isset($this->default_options['limit']) ? $this->default_options['limit'] : null),
			],
			[
				'type' => 'text',
				'col' => 4,
				'required' => false,
				'label' => $this->l('Selector of country field'),
				'hint' => 'This field is for country detection, because Web API works only with post codes in Lithuania', 
				'name' => self::MODULE_NAME . '_OPTIONS[input_country]',
				'suffix' => $this->inputSuffix(isset($this->default_options['input_country']) ? $this->default_options['input_country'] : null),
			],
			[
				'type' => 'text',
				'col' => 4,
				'required' => false,
				'label' => $this->l('Lithuania country ID'),
				'hint' => 'In "International > Locations > Countries" section search for Lithuania country ID', 
				'name' => self::MODULE_NAME . '_OPTIONS[input_country_lt_value]',
				'suffix' => $this->inputSuffix(isset($this->default_options['input_country_lt_value']) ? $this->default_options['input_country_lt_value'] : null),
			],
			[
				'type' => 'text',
				'col' => 4,
				'required' => false,
				'label' => $this->l('Selector of address field for autocompletion'),
				'hint' => 'When address is selected then "city" and "post code" fields are filled', 
				'name' => self::MODULE_NAME . '_OPTIONS[input_address]',
				'suffix' => $this->inputSuffix(isset($this->default_options['input_address']) ? $this->default_options['input_address'] : null),
			],
			[
				'type' => 'text',
				'col' => 4,
				'required' => false,
				'label' => $this->l('City field selector'),
				'name' => self::MODULE_NAME . '_OPTIONS[input_city]',
				'suffix' => $this->inputSuffix(isset($this->default_options['input_city']) ? $this->default_options['input_city'] : null),
			],
			[
				'type' => 'text',
				'col' => 4,
				'required' => false,
				'label' => $this->l('Post code field selector'),
				'name' => self::MODULE_NAME . '_OPTIONS[input_postcode]',
				'suffix' => $this->inputSuffix(isset($this->default_options['input_postcode']) ? $this->default_options['input_postcode'] : null),
			],
			[
				'type' => 'text',
				'col' => 4,
				'required' => false,
				'label' => $this->l('jQuery autocomplete version: 0 - auto, 1 - old, 2 - new'),
				'name' => self::MODULE_NAME . '_OPTIONS[autocomplete_version]',
				'suffix' => $this->inputSuffix(isset($this->default_options['autocomplete_version']) ? $this->default_options['autocomplete_version'] : null),
			],
		];
		
		$is_trial = $this->getOption('is_trial');
		if($is_trial){
			array_unshift($inputs, [
				'type' => 'html',
				'name' => $this->name . '_promo_text_warning',
				'class' => 'fixed-width-xs',
				'html_content' => $this->adminDisplayWarning('You are using <a href="https://postit.lt" target="_blank">postit.lt</a> trial key, so the following promotional text is added below postcode field.'
					. '<br /><a href="https://postit.lt/kontaktai/" target="_blank">Write to us</a> if you need more API requests or if You need promo free key.'
					. '<br /><br /><pre>' . htmlspecialchars($this->trial_text) . '</pre>' 
				),
			]);
		}
		
		$form = [
			'legend' => [
				'title' => $this->l('Settings'),
				'icon' => 'icon-cogs',
			],
			'input' => $inputs,
			'submit' => [
				'title' => $this->l('Save'),
			],
		];
		
		return ['form' => $form];
	}
	
	protected function inputSuffix($value)
	{
		if(!isset($value)){
			return null;
		}
		
		$script = '<a href="#" onclick="$(this).closest(\'.input-group\').find(\'input\').val(\''. htmlentities($value, ENT_QUOTES).'\'); return false; ">'.$this->l('Reset to default').'</a>';
		return $script;
	}

	public function getConfigFormValues()
	{
		$values = [
			self::MODULE_NAME . '_API_KEY' => Configuration::get(self::MODULE_NAME . '_API_KEY'),
		];
		
		$options = $this->getOptions();
		foreach($options as $key => $value){
			$values[self::MODULE_NAME . '_OPTIONS['.$key.']'] = $value;
		}
		
		return $values;
	}
	
	protected function getOptions()
	{
		$options = json_decode(Configuration::get(self::MODULE_NAME . '_OPTIONS'), true);
		if(!is_array($options)){
			$options = [];
		}
		return array_merge($this->default_options, $options);
	}
	
	protected function getOption($key, $default = null)
	{
		$options = $this->getOptions();
		return isset($options[$key]) ? $options[$key] : $default;
	}

	/*
	 * Save values
	 */
	protected function postProcess()
	{
		$output = '';
		$key = trim(Tools::getValue(self::MODULE_NAME . '_API_KEY', ''));
		$options = Tools::getValue(self::MODULE_NAME . '_OPTIONS', []);
		if(!is_array($options)){
			$options = [];
		}
		
		$options = array_intersect_key(array_merge($this->getOptions(), $options), $this->getOptions());
		$options['autocomplete_version'] = (int)$options['autocomplete_version'];
		$options['limit'] = (int)$options['limit'];
		if($options['limit'] <= 0){
			$options['limit'] = 1;
		}
		$options['input_country_lt_value'] = trim($options['input_country_lt_value']);
		
		$keyStatus = $this->getKeyStatus($key);
		if(!$keyStatus || empty($keyStatus['is_valid'])){
			$key = '';
			$output .= $this->displayError($this->l('API key is not valid.'));
		}
		else {
			$options['is_trial'] = !empty($keyStatus['is_trial']);
			$output .= $this->displayConfirmation($this->l($options['is_trial'] ? 'API trial mode enabled.' : 'API trial mode disabled.'));
		}
		
		Configuration::updateValue(self::MODULE_NAME . '_API_KEY', $key);
		Configuration::updateValue(self::MODULE_NAME . '_OPTIONS', json_encode($options));
		
		$output .= $this->displayConfirmation($this->l('The settings have been updated.'));
		
		return $output;
	}
	
	public function hookHeader()
	{
		$key = Configuration::get(self::MODULE_NAME . '_API_KEY');
		$controller = strtolower(str_ireplace('controller', '', get_class($this->context->controller)));
		
		if(empty($key) || !in_array($controller, ['order', 'address', 'orderopc', 'auth'])){
			return;
		}
		
		$options = json_decode(Configuration::get(self::MODULE_NAME . '_OPTIONS'), true);
		if(!is_array($options)){
			$options = [];
		}
		
		$data = [
			'url' => self::URL_V1,
			'version' => 'v1',
			'args' => [
				'key' => $key,
				'limit' => isset($options['limit']) ? (int)$options['limit'] : 10,
			],
			'config' => [
				'prefix' => isset($options['prefix']) ? $options['prefix'] : '',
				'label_suffix' => isset($options['label_suffix']) ? $options['label_suffix'] : '',
			],
			'params' => [
				'country' => isset($options['input_country']) ? $options['input_country'] : '',
				'country_lt_value' => isset($options['input_country_lt_value']) ? $options['input_country_lt_value'] : '',
				'address' => isset($options['input_address']) ? $options['input_address'] : '',
				'city' => isset($options['input_city']) ? $options['input_city'] : '',
				'municipality' => isset($options['input_municipality']) ? $options['input_municipality'] : '',
				'postcode' => isset($options['input_postcode']) ? $options['input_postcode'] : '',
				'is_trial' => isset($options['is_trial']) ? (bool)$options['is_trial'] : true,
				'trial_text' => '',
			],
		];
		
		if(!empty($data['params']['is_trial'])){
			$data['params']['trial_text'] = '<span style="font-size:80%">'.$this->trial_text.'</span>';
		}

		Media::addJsDef([$this->name . '_data' => $data]);
		$this->addJS($this->_path . 'views/js/', (int)$options['autocomplete_version']);
	}
	
	protected function addJS($path, $autocompleteVersion = 0){
		if(version_compare(_PS_VERSION_, '1.7.0') >= 0){
			$jsFile = $autocompleteVersion == 1 ? 'postit_ps16.js' : 'postit.js';
			$this->context->controller->addJqueryUi('ui.autocomplete');
			if ($uri = $this->context->controller->getAssetUriFromLegacyDeprecatedMethod($path . $jsFile)) {
				$this->context->controller->registerJavascript(sha1($uri), $uri, ['position' => 'bottom', 'priority' => 110]);
			}
		}
		else {
			$jsFile = $autocompleteVersion == 2 ? 'postit.js' : 'postit_ps16.js';
			$this->context->controller->addJquery();
			$this->context->controller->addJqueryPlugin('autocomplete');
			$this->context->controller->addJS($path . $jsFile);
		}
	}
	
	public function getKeyStatus($key){
		if(empty($key)){
			return false;
		}
		
		$response = file_get_contents(str_replace('{KEY}', $key, self::URL_CHECK));
		if(!$response){
			return false;
		}
		
		$data = json_decode($response, true);
		if(!is_array($data)){
			return false;
		}
		
		return $data;
	}
}